"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Reducer_1 = require("../../src/redux-retro/Reducer");
var Actions_1 = require("../../src/redux-retro/Actions");
describe('Reducers', function () {
    var reducer;
    var initialState = 'initial state';
    var TestActions = /** @class */ (function (_super) {
        __extends(TestActions, _super);
        function TestActions() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TestActions.prototype.a = function () {
            return 'A';
        };
        TestActions.prototype.b = function () {
            return 'B';
        };
        return TestActions;
    }(Actions_1.Actions));
    var testActions;
    beforeEach(function () {
        testActions = new TestActions({});
        reducer = Reducer_1.createReducer(initialState);
    });
    describe('Initial state', function () {
        describe('When a reducer is called with no state passed in', function () {
            var newState;
            beforeEach(function () {
                newState = reducer(undefined, {
                    type: '@test',
                    payload: null
                });
            });
            it('should return the reducer\'s initial state', function () {
                expect(newState).toBe(initialState);
            });
        });
    });
    describe('Handling dispatched actions', function () {
        describe('When the reducer is called with an action type bound to it', function () {
            var expectedState = 'state that the bound action returns';
            var newState;
            beforeEach(function () {
                reducer.bindAction(TestActions.prototype.a, function (state, action) {
                    return expectedState;
                });
                newState = reducer(initialState, {
                    type: 'A',
                    payload: null
                });
            });
            it('should return the state that the bound function returns', function () {
                expect(newState).toBe(expectedState);
            });
        });
        describe('When the reducer is called with an action type not bound to it', function () {
            var otherActionState = 'other action state';
            var passedInState = 'passedInState';
            var newState;
            beforeEach(function () {
                reducer.bindAction(TestActions.prototype.a, function (state, action) {
                    return otherActionState;
                });
                newState = reducer(passedInState, {
                    type: 'B',
                    payload: null
                });
            });
            it('should return the state passed to it', function () {
                expect(newState).toBe(passedInState);
            });
        });
        describe('When a reducer is bound to action types A and B, and action type A is dispatched', function () {
            var reducerFunctionForActionA = jasmine.createSpy('reducerFunctionForActionA');
            var reducerFunctionForActionB = jasmine.createSpy('reducerFunctionForActionB');
            beforeEach(function () {
                reducer
                    .bindAction(TestActions.prototype.a, reducerFunctionForActionA)
                    .bindAction(TestActions.prototype.b, reducerFunctionForActionB);
                reducer(initialState, {
                    type: 'A',
                    payload: null
                });
            });
            it('should call the bound reducer function for action A', function () {
                expect(reducerFunctionForActionA).toHaveBeenCalled();
            });
            it('should NOT call the bound reducer function for action B', function () {
                expect(reducerFunctionForActionB).not.toHaveBeenCalled();
            });
        });
    });
});
//# sourceMappingURL=Reducer.specs.js.map