"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Actions = /** @class */ (function () {
    function Actions(store) {
        this.store = store;
        this.reconfigureActions();
    }
    Actions.prototype.getState = function () {
        return this.store.getState();
    };
    Actions.prototype.convertMethodNameToActionType = function (methodName) {
        return methodName.replace(/[a-z]([A-Z])/g, function (i) {
            return i[0] + "_" + i[1].toLowerCase();
        }).toUpperCase();
    };
    Actions.prototype.checkIfPromise = function (obj) {
        return !!obj && (typeof obj === 'object' || typeof obj === 'function') && typeof obj.then === 'function';
    };
    Actions.prototype.generateWrapperFunction = function (member) {
        var _this = this;
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var payload = member.apply(_this, args);
            if (payload !== undefined && !_this.checkIfPromise(payload)) {
                _this.dispatch({
                    type: member.actionType,
                    payload: payload
                });
            }
            return payload;
        };
    };
    Actions.prototype.reconfigureActions = function () {
        var _this = this;
        var prototype = this.constructor.prototype;
        var members = Object.getOwnPropertyNames(prototype).filter(function (p) { return p !== 'constructor'; });
        members.forEach(function (memberName) {
            var member = prototype[memberName];
            if (typeof member == 'function' && prototype.hasOwnProperty(memberName)) {
                var actionType = _this.convertMethodNameToActionType(memberName);
                member.actionType = actionType;
                _this[memberName] = _this.generateWrapperFunction(member);
            }
        });
    };
    Actions.prototype.dispatch = function (action) {
        this.store.dispatch(action);
    };
    return Actions;
}());
exports.Actions = Actions;
//# sourceMappingURL=Actions.js.map